package batch;

import java.io.IOException;
import java.util.Date;

import org.neo4j.graphdb.Direction;
import org.neo4j.graphdb.Node;
import org.neo4j.graphdb.Relationship;
import org.neo4j.graphdb.Transaction;
import org.neo4j.index.IndexHits;
import org.neo4j.index.lucene.LuceneIndexService;
import org.neo4j.kernel.EmbeddedGraphDatabase;

import utilities.User;

/**
 * Elimina dal grafo le categorie che non hanno valore semantico
 * e che sono riportate nel file precedentemente generato da EstrazioneCatagorieInterne
 * 
 * */
public class EliminaCategoriaSpecifica {

	public static void main(String argc[]) throws IOException {
		new EliminaCategoriaSpecifica();
	}


	private EmbeddedGraphDatabase graph;
	private LuceneIndexService index;


	private int manipolazioni=0;
	private Transaction tx;


	public EliminaCategoriaSpecifica() throws IOException{
		System.out.println("Avvio il database e apro la transazione...");
		graph=new EmbeddedGraphDatabase( "wp_semantico/" );
		tx=graph.beginTx();

		System.out.println("Fatto. Avvio l'indice...");
		index = new LuceneIndexService(graph);
		System.out.println("indice avviato.");

		try{

			System.out.println("Fatto, inizio il lavoro");
			String letta;
			int riga=0;
			int manipolazioni=0;
			while(!(letta=User.chiediStringa("cosa elimino?")).equals("")){
				riga++;
				manipolazioni++;
				//per ogni nodo chiudo e riapro la transazione
				//questo perchÃ© alcuni nodi hanno moltissime relazioni e darebbero problemi i "nodi fantasma"
				tx.success();
				tx.finish();
				tx=graph.beginTx();

				System.out.println(letta);
				String nome=letta;
				IndexHits<Node> nodi = index.getNodes("name", nome);

				for(Node n:nodi){
					System.out.println("nodi.size:"+nodi.size()+" properietà : "+n.getPropertyKeys());
				}

				nodi = index.getNodes("name", nome);

				if(!nodi.hasNext()){
					System.out.println("Nessuna categoria con nome="+nome+" alla riga "+riga);
					continue;
				}
				nodi = index.getNodes("name", nome);
				//questo if serve a supplire l'errore dell'indice che restituisce una 
				//dimensione sbagliata se si chiama il metodo size()
				if(nodi.hasNext()? !nodi.hasNext():false){
					Node el=index.getSingleNode("name", nome);
					if (el==null){
						System.out.println("ERRORE NULL");
						continue;
					}
					//prima devo rimuovere le relazioni attaccate
					while(el.hasRelationship())
						for(Relationship r:el.getRelationships(Direction.BOTH)){
							System.out.println("elimino relazione "+r.getEndNode().getProperty("name")+" - "+r.getStartNode().getProperty("name"));
							r.delete();
							manipolazioni++;
							if(manipolazioni%100==0){
								//ogni 100 righe svuoto e ricomincio la transazione
								tx.success();
								tx.finish();
								tx=graph.beginTx();
							}
							//tentativo:interrompi il ciclo, che Ã¨ fasullo:non si itera su una cosa che si modifica
							break;

						}
					el.delete();
				}
				nodi = index.getNodes("name", nome);
				if(nodi.hasNext()){
					String scelta=User.scegliOpzione("pagina;categoria", "cosa elimino?");
					nodi = index.getNodes("name", nome);
					Node el = null;
					if(scelta.equals("pagina")){
						for(Node n:nodi) if(n.hasProperty("idpag")) el=n;
					}
					if(scelta.equals("categoria")){
						for(Node n:nodi) if(n.hasProperty("idcat")) el=n;
					}
					if (el==null){
						System.out.println("ERRORE NULL");
						continue;
					}
					//prima devo rimuovere le relazioni attaccate
					while(el.hasRelationship())
						for(Relationship r:el.getRelationships(Direction.BOTH)){
							System.out.println("elimino relazione "+r.getEndNode().getProperty("name")+" - "+r.getStartNode().getProperty("name"));
							r.delete();
							manipolazioni++;
							if(manipolazioni%100==0){
								//ogni 100 righe svuoto e ricomincio la transazione
								tx.success();
								tx.finish();
								tx=graph.beginTx();
							}
							//tentativo:interrompi il ciclo, che Ã¨ fasullo:non si itera su una cosa che si modifica
							break;

						}
					el.delete();
				}


			}

		}
		catch(Exception e){
			System.err.println("eccezione catturata:");
			e.printStackTrace();
		}
		finally{
			tx.success();
			tx.finish();
			graph.shutdown();
			System.out.println("FINE, programma terminato...");
			System.exit(1);
		}
	}


}
