package batch;

import java.io.IOException;

import javax.swing.JOptionPane;

import org.neo4j.graphdb.Direction;
import org.neo4j.graphdb.DynamicRelationshipType;
import org.neo4j.graphdb.GraphDatabaseService;
import org.neo4j.graphdb.Node;
import org.neo4j.graphdb.Relationship;
import org.neo4j.graphdb.RelationshipType;
import org.neo4j.graphdb.Transaction;
import org.neo4j.index.IndexHits;
import org.neo4j.index.IndexService;
import org.neo4j.index.lucene.LuceneIndexService;
import org.neo4j.kernel.EmbeddedGraphDatabase;

import utilities.User;


/**
 * Estrae un grafo contenente le categorie che non hanno un valore semantico ma 
 * servono all'organizzazione interna del progetto en.wikipedia.
 * NOTA: alcune categorie -individuate manualmente- sono escluse dalla selezione, 
 * perché hanno in realtà un valore semnatico pur essendo incluse in Wikipedia administration
 * */
public class EstrazioneCategorieInterne {

	public static void main(String argc[]) throws IOException{
		EstrazioneCategorieInterne e=new EstrazioneCategorieInterne();
	}

	private EmbeddedGraphDatabase graphDa;
	private EmbeddedGraphDatabase graphA;
	private LuceneIndexService indexDa;
	private LuceneIndexService indexA;

	private int manipolazioni=0;
	private Transaction txDa;
	private Transaction txA;
	private Scrittore scr;
	private boolean esportaPagine;

	public EstrazioneCategorieInterne() throws IOException{


		System.out.println("apro il database di partenza");
		this.graphDa = new EmbeddedGraphDatabase( "neo4j-db/" );
		String estrai=User.chiediStringa("nome della categorie da estrarre (generalmente Wikipedia administration)");

		try {
			scr=new Scrittore("export_"+estrai+".txt");
		} catch (IOException e) {
			e.printStackTrace();
			System.exit(1);
		}



		System.out.println("apro il database di destinazione");
		this.graphA = new EmbeddedGraphDatabase( "sample_"+estrai+"/" );
		int depth=User.chiediNumero("profondità dell'estrazione");

		System.out.println("apro l'indice di partenza");		
		this.indexDa = new LuceneIndexService( graphDa );

		System.out.println("apro l'indice di destinazione");		
		this.indexA = new LuceneIndexService( graphA );



		System.out.println("apro la transazione di partenza");
		this.txDa = graphDa.beginTx();

		System.out.println("apro la transazione di destinazione");
		this.txA = graphA.beginTx();

		IndexHits<Node> nodi = indexDa.getNodes("name", estrai);

		String direzione=User.scegliOpzione("entranti;uscenti;entrambe","direzione delle relazioni da seguire");
		Direction dir=null;
		if(direzione.equals("entranti")) dir=Direction.INCOMING;
		if(direzione.equals("uscenti")) dir=Direction.OUTGOING;
		if(direzione.equals("entrambe")) dir=Direction.BOTH;

		if(User.scegliOpzione("solo categorie;pagine e categorie","scegli cosa esportare").equals("solo categorie"))
			esportaPagine=false;
		else
			esportaPagine=true;


		if(nodi.size()==1){
			Node nuovoA=graphA.createNode();

			Node originale=nodi.next();
			for(String prop:originale.getPropertyKeys()) nuovoA.setProperty(prop, originale.getProperty(prop));
			salvaRamo(originale,nuovoA,depth,dir,DynamicRelationshipType.withName( "SUBCATEGORYOF"));

			System.out.println("finito, salvo e chiudo...");
			txDa.success();txA.success();
			txDa.finish();txA.finish();
			graphDa.shutdown();graphA.shutdown();
			scr.chiudi();
			System.out.println("tutto chiuso");
		}

		if(nodi.size()==0){
			System.out.println("errore, nessun nodo con con proprietà name="+estrai);
			txDa.finish();txA.finish();
			graphDa.shutdown();graphA.shutdown();
			scr.chiudi();
			System.exit(1);
		}

		if(nodi.size()>1){
			System.out.println("errore, ci sono "+nodi.size()+" nodi con con proprietà name="+estrai);
			String scelta=User.scegliOpzione("esportali tutti;esporta solo categorie;esporta solo pagine;ESCI","Cosa faccio con i nodi multipli");
			if (scelta.equals("ESCI")){
				System.out.println("esco...");
				txDa.finish();txA.finish();
				graphDa.shutdown();graphA.shutdown();
				System.exit(1);
			}

			if (scelta.equals("esportali tutti")){
				for(Node n:nodi){
					Node nuovo=graphA.createNode();
					for(String prop:n.getPropertyKeys()) nuovo.setProperty(prop, n.getProperty(prop));

					salvaRamo(n,nuovo,depth,dir,DynamicRelationshipType.withName("SUBCATEGORYOF"));
				}

			}

			if (scelta.equals("esporta solo categorie")){
				for(Node n:nodi)
					if (n.hasProperty("idcat")) {
						Node nuovo=graphA.createNode();
						for(String prop:n.getPropertyKeys()) nuovo.setProperty(prop, n.getProperty(prop));
						salvaRamo(n,nuovo,depth,dir,DynamicRelationshipType.withName( "SUBCATEGORYOF"));					}
			}


			if (scelta.equals("esporta solo pagine")){
				for(Node n:nodi)
					if (n.hasProperty("idpag")){
						Node nuovo=graphA.createNode();
						for(String prop:n.getPropertyKeys()) nuovo.setProperty(prop, n.getProperty(prop));

						salvaRamo(n,nuovo,depth,dir,DynamicRelationshipType.withName( "SUBCATEGORYOF"));					
					}
			}

			System.out.println("finito, salvo e chiudo...");
			txDa.success();txA.success();
			txDa.finish();txA.finish();
			graphDa.shutdown();graphA.shutdown();
			System.out.println("tutto chiuso");
			scr.chiudi();
		}
	}

	private void salvaRamo(Node nodeDa,Node nodeA, int depth,Direction direzione,RelationshipType tipo) throws IOException {
		if (depth==0) return;

		
		
//		controlli per non esportare categorie che non sono riferite al progetto wikipedia stesso
		//ma sono sottocategorie di Wikipedia administration

		//la categorie Parent categories NON deve essere esportata (va esportata la categoria ma non le categorie di appartenenza)
		if (((String) nodeDa.getProperty("name", "")).equalsIgnoreCase("Parent categories")){
			System.out.println("PARENT CATEGORIES, lo salto!");
			return;
		}
		//se questo nodo è l'elenco delle categorie da controllare
		// mi interrompo nella ricorsione
		if (((String) nodeDa.getProperty("name", "")).equals("Wikipedia categories in need of attention")) {
			System.out.println("NEED OF ATTENTION, lo salto!");
			return;
		}
		
		
//		se questo nodo è l'elenco delle categoriesottopopolate
		// mi interrompo nella ricorsione
		if (((String) nodeDa.getProperty("name", "")).equals("Underpopulated categories")) {
			System.out.println("UNDERPOPULATED, lo salto!");
			return;
		}
		
//		se questo nodo è l'elenco delle categorie sovrappopolate
		// mi interrompo nella ricorsione
		if (((String) nodeDa.getProperty("name", "")).equals("Very large categories")) {
			System.out.println("OVERPULATED, lo salto!");
			return;
		}
		
	
//		se questo nodo è l'elenco delle categorie che richiedono maggiore diffusione
		// mi interrompo nella ricorsione
		if (((String) nodeDa.getProperty("name", "")).equals("Categories requiring diffusion")) {
			System.out.println("REQUIRE MORE DIFFUSION, lo salto!");
			return;
		}
		
		
		if (((String) nodeDa.getProperty("name", "")).equals("Christianity Portal pages")) {
			System.out.println("Christianity Portal pages, lo salto!");
			return;
		}
		
		
		
		if (((String) nodeDa.getProperty("name", "")).equals("Organization categories to adjust")) {
			System.out.println("Organization categories to adjust, lo salto!");
			return;
		}
		
		if (((String) nodeDa.getProperty("name", "")).equals("WikiProjects")) {
			System.out.println("WikiProjects, lo salto!");
			return;
		}
		
		if (((String) nodeDa.getProperty("name", "")).equals("Categories which are included in the JEL classification codes")) {
			System.out.println("Categories which are included in the JEL classification codes, lo salto!");
			return;
		}
		
		if (((String) nodeDa.getProperty("name", "")).equals("Underpopulated United States categories")) {
			System.out.println("Underpopulated United States categories, lo salto!");
			return;
		}
		
		
		
		if(nodeA.hasProperty("name")) indexA.index(nodeA, "name", nodeA.getProperty("name"));
		if(nodeA.hasProperty("idcat")) indexA.index(nodeA, "idcat", nodeA.getProperty("idcat"));
		if(nodeA.hasProperty("idpag")) indexA.index(nodeA, "idpag", nodeA.getProperty("idpag"));
		if(nodeA.hasProperty("idcat")){
			scr.scrivi("cat "+nodeA.getProperty("idcat")+" "+nodeA.getProperty("name"));	
		}


		if(nodeA.hasProperty("name")) System.out.println("ricevo il nodo "+nodeA.getProperty("name"));


		for(Relationship r:nodeDa.getRelationships(tipo, direzione)){
			manipolazioni++;
			if(manipolazioni%1000==0){
				txA.success();
				txA.finish();
				txA=graphA.beginTx();
				txDa.success();
				txDa.finish();
				txDa=graphDa.beginTx();
			}

			Node altro=r.getOtherNode(nodeDa);

			

//			se questo nodo è un elenco delle categorie sottopopolate
			//ma non sto aggiungendo un nodo delle categorie sottopopolate, mi interrompo nella ricorsione
//			if (((String) nodeDa.getProperty("name", "")).equalsIgnoreCase(".*[Uu]nderpopulated.*") &&
//					!((String) altro.getProperty("name", "")).matches(".*[Uu]nderpopulated.*")) {
//				System.out.println("UNDERPOPULATED ma non contiene UNDERPOPULATED, lo salto!");
//				continue;
//			}

			Node creato=graphA.createNode();

			if(direzione.equals(Direction.INCOMING))
				creato.createRelationshipTo(nodeA, tipo);
			else
				nodeA.createRelationshipTo(creato, tipo);

			for(String prop:altro.getPropertyKeys())
				creato.setProperty(prop, altro.getProperty(prop));

			salvaRamo(altro,creato,depth-1,direzione,tipo);
		}

		//esporto anche le pagine ?
		if(this.esportaPagine){
			System.out.println("clono anche le pagine");	

			for(Relationship r:nodeDa.getRelationships(DynamicRelationshipType.withName( "BELONGSTO"), Direction.INCOMING)){
				manipolazioni++;
				if(manipolazioni%1000==0){
					txA.success();
					txA.finish();
					txA=graphA.beginTx();
					txDa.success();
					txDa.finish();
					txDa=graphDa.beginTx();
				}

				Node creato=graphA.createNode();
				Node altro=r.getOtherNode(nodeDa);
				if(direzione.equals(Direction.INCOMING))
					creato.createRelationshipTo(nodeA, tipo);
				else
					nodeA.createRelationshipTo(creato, tipo);

				for(String prop:altro.getPropertyKeys())
					creato.setProperty(prop, altro.getProperty(prop));

				//non salvo il ramo perché una pagina non contiene della categorie 
				//o altre pagine e per ora non contemplo la possibilità di risalire i rami dalle pagine
				//salvaRamo(altro,creato,depth-1,direzione,tipo);

				//voglio però segnare le pagien nel log

				if(nodeA.hasProperty("idpag")){
					scr.scrivi("pag "+nodeA.getProperty("idpag")+" "+nodeA.getProperty("name"));	
				}
			}


		}




	}



}
