package batch;

import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.MalformedURLException;
import java.net.URL;


/**
 * Un lettore fa riferimento a un file o una cartella acceduta in locale o via http
 * e permette di ottenere altri lettori utilizzando indirizzi relativi
 * */
public class Lettore{

	private String risorsa;
	private BufferedReader in;
	private boolean isURL;
	private boolean isFolder;
	private int riga;



	public static void main(String argc[]) throws IOException{
		//		Lettore l=new Lettore("mio_gioco/edit.dat");
		//		System.out.println(l);
		//		Lettore d=l.creaDa("test.map");
		//		System.out.println(d);

		Lettore f=new Lettore("mio_gioco/cells");
		System.out.println(f);
		System.out.println(f.isFolder);

		Lettore nf=f.creaDa("city.png");
		System.out.println(nf);
		System.out.println(nf.isFolder);

	}


	/**
	 * Crea un lettore che punti al file, alla cartella o all'URL indicati
	 * @param indirizzo il percorso assoluto o relativo nel filesystem o l'URL
	 * */
	public Lettore(String indirizzo) {
		//il lettore deve fare riferimento a un file o un URL, 
		//e deve poter creare un lettore partendo da un indirizzo relativo
		//gestendo le diverse scritture dei file e semplificandone l'uso

		this.risorsa=indirizzo;

		if(risorsa.startsWith("http://")) {
			this.isURL=true;
			if (risorsa.endsWith("/"))isFolder=true;

		}
		else{
			this.isURL=false;
			risorsa=(new File(indirizzo)).getAbsolutePath();
			if (new File(risorsa).isDirectory()){isFolder=true;return;}

		}
	}

	public Lettore(File cartella) throws IOException {
		this(cartella.getAbsolutePath());
	}


	/**
	 * Restituisce un oggetto di tipo BufferedReader per la risorsa che rappresenta
	 * La risorsa potrebbe non essere accessibile o essere una cartella, quindi posso lanciare delle eccezioni
	 * */
	public BufferedReader getReader() throws MalformedURLException, IOException{
		if (in!=null) return in;
		if (this.isURL){
			return new BufferedReader(new InputStreamReader((new URL(risorsa)).openStream()));
		}
		else{

			FileInputStream fstream = new FileInputStream(risorsa);
			DataInputStream dis = new DataInputStream(fstream);
			in=new BufferedReader(new InputStreamReader(dis));
			return in;	
		}

	}

	/**
	 * Apre un BufferedReader, se non era già aperto, e legge la prima  stringa dal file
	 * a ogni chiamata legge la successiva stringa o restituisce null se arriva alla fine
	 * */
	public String leggi() throws IOException{
		if(in==null){
			if (this.isURL){
				in = new BufferedReader(new InputStreamReader((new URL(risorsa)).openStream()));
			}
			else{

				FileInputStream fstream = new FileInputStream(risorsa);
				DataInputStream dis = new DataInputStream(fstream);
				in = new BufferedReader(new InputStreamReader(dis));	
			}
		}
		riga++;
		return in.readLine();

	}

	public String toString(){
		if (!this.isURL && this.isFolder){
			return risorsa+File.separatorChar;
		}
		return risorsa;
	}

	/**
	 * crea un altro Lettore partendo da un percorso che può essere relativo a quello attuale
	 * */
	public Lettore creaDa(String indirizzo) {
		//vari casi possibili:

		//È un URL completo
		if(indirizzo.startsWith("http://")) return new Lettore(indirizzo);
		//È un percorso assoluto di file su filesystem tipo UNIX
		if (File.separatorChar=='/' && indirizzo.matches("/.+")) return new Lettore(indirizzo);
		//È un percorso assoluto di file su filesystem di tipo DOS (Windows)
		if(File.separatorChar=='\\' && indirizzo.matches("[a-zA-Z]:\\.+")) return new Lettore(indirizzo);

		//la parte importante:indirizzo relativo

		//il mio indirizzo è un URL o in un filesystem unix
		if (this.isURL() || risorsa.startsWith("/")){
			String folder;
			if (isFolder)
				folder=risorsa;
			else
				folder=risorsa.substring(0, risorsa.lastIndexOf('/'));

			//converto le barre \ in /
			return new Lettore(folder+"/"+indirizzo.replace('\\', '/'));
		}

		//il mio indirizzo è un file in un filesystem windows
		if (risorsa.matches("[a-zA-Z]:\\.+")){
			String folder;
			if (isFolder)
				folder=risorsa;
			else
				folder=risorsa.substring(0, risorsa.lastIndexOf('\\'));
			return new Lettore(folder+"\\"+indirizzo.replace('/', '\\'));
		}

		//se non sono ancora ritornato, mi trovo qui e c'è stato un errore
		//TODO come gestirlo?
		return null;

	}

	public boolean isURL() {
		return isURL;
	}


	public int getNumeroRiga() {
		return this.riga;
	}

	/**
	 * restituisce il percorso relativo a quello corrente, se possibile, di un percorso dato 
	 * altrimenti quello assoluto
	 * @throws IOException se il file non esiste
	 * */
	public String getPercorsoRelativo(String percorso) throws IOException {
		//mi basta controllare che il percorso assoluto della cartella mio file
		//contenga quello della cartella del percorso dato
		String percorsoAssoluto=this.creaDa(percorso).toString();

		String cartellaAssoluta="";

		if(!this.isFolder && (this.isURL|| File.separatorChar=='/')) 
			cartellaAssoluta=risorsa.substring(0, risorsa.lastIndexOf('/'));
		if(!this.isFolder && (!this.isURL && File.separatorChar=='\\'))
			cartellaAssoluta=risorsa.substring(0, risorsa.lastIndexOf('\\'));

		//aggiungo +1 per lo \ o lo /, che devono essere eliminati
		if (this.risorsa.startsWith(cartellaAssoluta)) 
			return percorsoAssoluto.substring(cartellaAssoluta.length()+1);
		else 
			return percorsoAssoluto;
	}

	public void chiudi() throws IOException {
		in.close();
	}


	public boolean isCartella() {
		return isFolder;
	}

}
