package batch;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.neo4j.graphdb.DynamicRelationshipType;
import org.neo4j.graphdb.Node;
import org.neo4j.graphdb.RelationshipType;
import org.neo4j.index.IndexHits;
import org.neo4j.index.IndexService;
import org.neo4j.index.lucene.LuceneIndexBatchInserter;
import org.neo4j.index.lucene.LuceneIndexBatchInserterImpl;
import org.neo4j.kernel.impl.batchinsert.BatchInserter;
import org.neo4j.kernel.impl.batchinsert.BatchInserterImpl;

import batch.Lettore;

public class TrasferimentoInNeo4j {


	public static void main(String argc[]) throws IOException{


		//creo l'oggetto per inserire i dati senza transazioni (modalità batch)
		BatchInserter inserter = new BatchInserterImpl( "wikipedia_sample/", BatchInserterImpl.loadProperties( "neo4j.props" ) );
		//creo il servizio di indicizzazione per i nodi
		LuceneIndexBatchInserter indexService = new LuceneIndexBatchInserterImpl( inserter );


		//notifico l'avvio del programma (per testare che la mail arrivi)
		try {
			SimpleMail.invia("avviato", "trasferimento avviato", "Far88@msn.com");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

//		creo un log degli errori
		Scrittore err=new Scrittore ("errori.log");

//		inserisco le categorie nel grafo
		Lettore l=new Lettore("categorie.txt");
		String letta;

//		creo la mappa per memorizzare le proprietà dei nodi
		Map<String,Object> properties = new HashMap<String,Object>();
		System.out.println("inizio la lettura del file categorie.txt e il suo salvataggio nel db a grafi...");
		while ((letta=l.leggi())!=null)
		{
			String[] parti = letta.split(">");

			if(parti.length<2){
				err.scrivi("errore, categoria "+parti[0]+" senza nome");
				continue;
			}
			if(parti[1].isEmpty()){
				err.scrivi("errore, categoria "+parti[0]+" con nome vuoto");
				continue;
			}

			parti[1]=parti[1].replace("_", " ");
			parti[1]=parti[1].replace("&amp;", "&");
			parti[1]=parti[1].replace("[^-a-zA-Z 0-9_&!?^=$()\\[\\]{}\\\\/]", "");
			System.out.println("memorizzo ("+parti[0]+") "+parti[1]);
			properties.clear();

			properties.put("idcat", new Integer(parti[0]));
			properties.put("name", parti[1]);

			long node = inserter.createNode( properties );

			indexService.index( node, "idcat", new Integer(parti[0]));   
			indexService.index( node, "name", parti[1]);   
		}

		l.chiudi();

		System.out.println("ho memorizzato e indicizzato tutte le categorie nel grafo");
//		optimize the index
		indexService.optimize();


		System.out.println("indice ottimizzato, ora leggo le appartenenze alle categorie");




		//ora memorizzo i nomi e gli id delle pagine
		l=new Lettore("articles.txt");
int esegui=0;
		while ((letta=l.leggi())!=null){
			esegui++;
			if (esegui%50!=0) continue;
			System.out.println(letta);
			String[] parti = letta.split(">");
			if(parti.length<2){
				err.scrivi("errore, articolo "+parti[0]+" senza nome");
				continue;
			}
			if(parti[1].isEmpty()){
				err.scrivi("errore, articolo "+parti[0]+" con nome vuoto");
				continue;
			}
			parti[1]=parti[1].replace("_", " ");	
			parti[1]=parti[1].replace("&amp;", "&");
			parti[1]=parti[1].replace("[^-a-zA-Z 0-9_&!?^=$()\\[\\]{}\\\\/]", "");

			//memorizzo le proprietà del nodo
			properties.clear();

			properties.put("idpag", new Integer(parti[0]));
			properties.put("name", parti[1]);

			long nodoArticolo = inserter.createNode( properties );

			inserter.setNodeProperties(nodoArticolo, properties);

			indexService.index( nodoArticolo, "idpag", new Integer(parti[0]));   
			indexService.index( nodoArticolo, "name", parti[1]);   
		}

		l.chiudi();

//		optimize the index
		indexService.optimize();

		//memorizzo l'appartenenza delle pagine e delle categorie a altre pagine o categorie


		l=new Lettore("links.txt");
		int contatore=0;
		IndexService serv = indexService.getIndexService();
		while ((letta=l.leggi())!=null){
			properties.clear();
			String[] parti = letta.split(">");
			parti[1]=parti[1].replace("_", " ");
			parti[1]=parti[1].replace("&amp;", "&");
			parti[1]=parti[1].replace("[^-a-zA-Z 0-9_&!?^=$()\\[\\]{}\\\\/]", "");
			contatore++;
			if (contatore%10000==0){System.out.println(parti[0]+" "+parti[1]);}


			//trovo il nodo del contenitore

			long contenitore = 0;
			try{
				contenitore=indexService.getSingleNode("name", parti[1]);
			}
			catch(Exception e){
				//esistono pagine e categorie con quel nome, dunque individuo la categoria 
				IndexHits<Node> possibili = serv.getNodes("name", parti[1]);

				for(Node n:possibili){
					if(n.hasProperty("idcat")) {
						contenitore=indexService.getSingleNode("idcat", n.getProperty("idcat"));
						break;
					}
				}
				err.scrivi("errore, nome "+parti[1]+" non corrispondente a una categoria in links.txt alla riga "+contatore);
				continue;
			}


			//l'appartenenza che sto analizzando è di una categoria o un articolo?
			//basta vedere se l'ID è presente nell'indice come idcat o idpag

			Node categoria=serv.getSingleNode("idcat", new Integer(parti[0]));

			//se è null, allora non è una categoria e creo un arco collegato con BELONGSTO

			if(categoria==null){
				long nodoPagina=indexService.getSingleNode("idpag", parti[0]);


				if(nodoPagina==-1){
					err.scrivi("errore alla riga "+contatore+" di links.txt: "+parti[0]+" non è un ID di pagina valido, ma appartiene a "+parti[1]);
					continue;
				}

				if(contenitore==-1){
					err.scrivi("errore alla riga "+contatore+" di links.txt: "+parti[0]+" appartiene a "+parti[1]+" che però non esiste");
					continue;
				}
				if(nodoPagina==contenitore){
					err.scrivi("Errore, "+letta+"assegna una pagina a se stessa alla riga "+contatore+" di links.txt");
					continue;
				}
				inserter.createRelationship(nodoPagina, contenitore, DynamicRelationshipType.withName( "BELONGSTO" ),(Map<String,Object>) null );

				continue;
			}


			long cat=indexService.getSingleNode("idcat", new Integer(parti[0]));


			if (cat==-1){
				//una categoria è contenuta in se stessa, ignoro questo fatto ma lo sgno tra gli errori
				err.scrivi(parti[0]+" è un nome di categoria inesistente in links.txt alla riga "+contatore);
				continue;
			}

			if (cat==contenitore){
				//una categoria è contenuta in se stessa, ignoro questo fatto ma lo sgno tra gli errori
				err.scrivi(parti[0]+" ossia "+categoria.getProperty("name")+" è contenuta in "+parti[1] +" ma corrisponde alla categoria stessa ");
				continue;
			}

			if(contatore%1000==0) System.out.println(parti[0]+" ossia "+categoria.getProperty("name")+" è contenuto in "+parti[1]);
			if(contenitore==-1){
				err.scrivi(parti[0]+" ossia "+categoria.getProperty("name")+" è contenuta in "+parti[1] +" ma la categoria non esiste alla riga "+contatore+" di links.txt");
				continue;
			}
			
			inserter.createRelationship(cat, contenitore, DynamicRelationshipType.withName( "SUBCATEGORYOF" ),(Map<String,Object>) null );


		}

		l.chiudi();

		try {
			SimpleMail.invia("links.txt terminato", "ho finito di trasferire i link", "Far88@msn.com");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
//		optimize the index
		indexService.optimize();
		try {
			SimpleMail.invia("indice ottimizzato", "indice dei link ottimizzato, inizio a memorizzare gli articoli", "Far88@msn.com");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		err.chiudi();


		indexService.shutdown();
		inserter.shutdown();

		try {
			SimpleMail.invia("FATTO", "ho finito e chiudo", "Far88@msn.com");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}