package distanze;

import java.util.AbstractCollection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import org.neo4j.graphalgo.centrality.NetworkDiameter;
import org.neo4j.graphalgo.shortestpath.Dijkstra;
import org.neo4j.graphalgo.shortestpath.SingleSourceSingleSinkShortestPath;
import org.neo4j.graphalgo.shortestpath.std.DoubleAdder;
import org.neo4j.graphalgo.shortestpath.std.DoubleComparator;
import org.neo4j.graphalgo.shortestpath.std.DoubleEvaluator;
import org.neo4j.graphdb.Direction;
import org.neo4j.graphdb.DynamicRelationshipType;
import org.neo4j.graphdb.Node;
import org.neo4j.graphdb.Relationship;

import wrapper.GraphWrapper;



public class Misuratore {

	private Set<Node> arrivi =new HashSet<Node>();
	private GraphWrapper g;

	public Misuratore (Iterable<String> arrivi,GraphWrapper g){
		for(String a:arrivi){
			this.arrivi.add(g.getCategoria(a));
		}
		this.g=g;
	}

	/**
	 * Fornisce i percorsi tra una categoria o una 
	 * pagina e i nodi d'arrivo definiti con il costruttore
	 * Se il nodo corrisponde a una pagina, allora fornisce i 
	 * percorsi più brevi tra le sue categorie e i nodi d'arrivo,
	 * selezionando quello più corto tra i percorsi che partono dalle
	 * varie categorie della pagina a arrivano allo stesso nodo d'arrivo
	 * */
	public Set<List<Node>> getPercorsi(Node n){
		if(n.hasProperty("idcat")){
			Set<List<Node>> set=new HashSet<List<Node>>();
			for(Node a:this.arrivi){
				System.out.println("Ricerca percorso minimo per "+a.getProperty("name"));
				List<Node> prc = getPercorso(n, a);
				if(prc==null){
					System.out.println("ERRORE: nodo nullo");
					continue;
				}
				set.add(prc);
			}
			return set;
		}
		//è una pagina, misuro per ogni categoria di appartenenza la distanza dai nodi di arrivo
		Set<Node> partenze = g.getCategorieAppartenenza(n);


		Set<List<Node>> set=new HashSet<List<Node>>();
		for(Node a:this.arrivi){
			System.out.println("Ricerca percorso minimo per "+a.getProperty("name"));
			double distanza=Double.MAX_VALUE;
			List<Node> prcMinimo=null;
			for(Node p:partenze){
				List<Node> prcAtt=getPercorso(p,a);
				if(prcAtt==null)continue;
				double distAtt=prcAtt.size();
				if(distAtt<distanza){
					distanza=distAtt;
					prcMinimo=prcAtt;
				}
			}
			if(prcMinimo==null)continue;
			set.add(prcMinimo);
		}

		return set;	
	}


	/**
	 * Restituisce la distanza tra due CATEGORIE,
	 * Se non già presente, abbina alla partenza la proprietà FROMx dove x è il nome della categoria d'arrivo
	 * utilizzandola in seguito per non rifare il calcolo
	 * */
	public double getDistanza(Node partenza,Node arrivo){
		if (partenza==null || arrivo==null) return -1;
		//se la categoria di partenza ha la proprietà FROM... la uso
		//altrimenti la calcolo e la imposto
		if(partenza.hasProperty("FROM"+arrivo.getProperty("name"))){
			return Double.parseDouble(partenza.getProperty("FROM"+arrivo.getProperty("name")).toString());
		}
		try{
			SingleSourceSingleSinkShortestPath<Double> sp = new Dijkstra<Double>(
					0.0, partenza, arrivo, new Valutatore(null),
					new DoubleAdder(), new DoubleComparator(), Direction.BOTH,
					DynamicRelationshipType.withName( "SUBCATEGORYOF" ));

			if (sp.getCost()==null) return -1;
			double dist=sp.getPathAsNodes().size();
			for(Node n:sp.getPathAsNodes()){
				if(!n.hasProperty("FROM"+arrivo.getProperty("name")))
					n.setProperty("FROM"+arrivo.getProperty("name"),sp.getCost());
				dist=dist-1;
			}
			partenza.setProperty("FROM"+arrivo.getProperty("name"),sp.getCost());
			return sp.getPathAsNodes().size();
		}
		catch(Exception e){
			e.printStackTrace();
		}
		return -1;
	}
/**
 * Restituisce il percorso tra due nodi CATEGORIA (non pagine)
 * */
	public List<Node> getPercorso(Node partenza,Node arrivo){
		if (partenza==null || arrivo==null)
			return null;
		try{
			SingleSourceSingleSinkShortestPath<Double> sp = new Dijkstra<Double>(
					0.0, partenza, arrivo, new Valutatore(null),
					new DoubleAdder(), new DoubleComparator(), Direction.BOTH,
					DynamicRelationshipType.withName( "SUBCATEGORYOF" ));

			return sp.getPathAsNodes();
		}
		catch(Exception e){
			e.printStackTrace();
		}
		return null;
	}

	/**
	 * Restituisce l'elenco delle distanze del nodo pagina di partenza dai nodi di arrivo
	 * @return 
	 * */
	public HashMap<String, Double> getDistanze(Node partenza) {
		HashMap<String,Double> risultato=new HashMap<String,Double>();

		for(Node a:this.arrivi){
			
			double minDist=Double.MAX_VALUE;
			
			HashMap<String,Double> aggiunta=new HashMap<String,Double> ();
			
			for(Node p:g.getCategorieAppartenenza(partenza)){
				double dist=this.getDistanza(p, a);
				if (dist<0) continue;
				if (dist>minDist) continue;
				//se è uguale, me ne basta uno
				if(dist==minDist)continue;
				//sono qua, quindi è minore
				minDist=dist;
				aggiunta.clear();
				aggiunta.put(a.getProperty("name").toString(), dist);

			}
			risultato.putAll(aggiunta);
		}
		return risultato;
	}

	/**
	 * Restituisce l'elenco dei nodi d'arrivo
	 * */
	public  Set<Node> getArrivi() {
		return arrivi;
	}
}

class Valutatore extends DoubleEvaluator{

	public Valutatore(String costpropertyName) {
		super(costpropertyName);
	}

	public Double getCost(Relationship r, boolean b){
		return 1.0;

	}

}
